# Benchmark driver.
#
# Author::    Yutaka Yanoh <mailto:yanoh@users.sourceforge.net>
# Copyright:: Copyright (C) 2010-2012, OGIS-RI Co.,Ltd.
# License::   GPLv3+: GNU General Public License version 3 or later
#
# Owner::     Yutaka Yanoh <mailto:yanoh@users.sourceforge.net>

#--
#     ___    ____  __    ___   _________
#    /   |  / _  |/ /   / / | / /__  __/           Source Code Static Analyzer
#   / /| | / / / / /   / /  |/ /  / /                   AdLint - Advanced Lint
#  / __  |/ /_/ / /___/ / /|  /  / /
# /_/  |_|_____/_____/_/_/ |_/  /_/   Copyright (C) 2010-2012, OGIS-RI Co.,Ltd.
#
# This file is part of adlint-benchmark.
#
# adlint-benchmark is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# adlint-benchmark is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# adlnt-benchmark.  If not, see <http://www.gnu.org/licenses/>.
#
#++

module AdLint #:nodoc:

  module Benchmark
    def run(target_name, count, print_time = false)
      if print_time
        require "benchmark"
        ::Benchmark.bm(target_name.length + 5, "total") do |x|
          total = count.times.reduce(::Benchmark::Tms.new) { |t, i|
            t += x.report("#{target_name} ##{i + 1}") { _run(target_name) }
          }
          [total]
        end
      else
        count.times { _run(target_name) }
      end
    end
    module_function :run

    def _run(target_name)
      if target = AnalysisTarget.load(target_name)
        _nullify_console_output { target.analyze }
      else
        raise "no such target `#{target_name}'"
      end
    end
    module_function :_run

    def _nullify_console_output(&block)
      orig_stdout = $stdout.dup
      orig_stderr = $stderr.dup
      dev_null = File.open(File::NULL, "w")
      $stdout.reopen(dev_null)
      $stderr.reopen(dev_null)
      yield
    ensure
      $stdout.reopen(orig_stdout)
      $stderr.reopen(orig_stderr)
    end
    module_function :_nullify_console_output
  end

end
