# Analysis target.
#
# Author::    Yutaka Yanoh <mailto:yanoh@users.sourceforge.net>
# Copyright:: Copyright (C) 2010-2012, OGIS-RI Co.,Ltd.
# License::   GPLv3+: GNU General Public License version 3 or later
#
# Owner::     Yutaka Yanoh <mailto:yanoh@users.sourceforge.net>

#--
#     ___    ____  __    ___   _________
#    /   |  / _  |/ /   / / | / /__  __/           Source Code Static Analyzer
#   / /| | / / / / /   / /  |/ /  / /                   AdLint - Advanced Lint
#  / __  |/ /_/ / /___/ / /|  /  / /
# /_/  |_|_____/_____/_/_/ |_/  /_/   Copyright (C) 2010-2012, OGIS-RI Co.,Ltd.
#
# This file is part of adlint-benchmark.
#
# adlint-benchmark is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# adlint-benchmark is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# adlnt-benchmark.  If not, see <http://www.gnu.org/licenses/>.
#
#++

require "pathname"
require "stringio"
require "fileutils"

module AdLint #:nodoc:
module Benchmark #:nodoc:

  class AnalysisTarget
    def self.load(name)
      require(name_to_rb_fname(name))
      eval "#{name_to_rb_cname(name)}.new('#{name}')"
    rescue LoadError
      nil
    end

    def self.cleanpath(str)
      Pathname.new(str).cleanpath
    end
    private_class_method :cleanpath

    def initialize(name)
      @name = name
    end

    attr_reader :name

    def analyze
      patch_libraries do
        _analyze
      end
    end

    private
    def _analyze
      subclass_responsibility
    end

    def patch_libraries(&block)
      eval <<EOS
class ::File
  class <<self
    alias :_orig_open :open
    def open(path, mode = "r", perm = 0666, &block)
      path = path.to_s
      if content = #{self.class.name}::FILES[Pathname.new(path).cleanpath]
        if content == :new
          io = File.open(File::NULL, "w")
        else
          io = StringIO.new(content)
        end
        io.set_encoding(Encoding::UTF_8)
        if block_given?
          yield(io)
        else
          io
        end
      else
        _orig_open(path, mode, perm, &block)
      end
    end

    alias :_orig_exist? :exist?
    def exist?(path)
      path = path.to_s
      paths = #{self.class.name}::DIRS + #{self.class.name}::FILES.keys
      if paths.include?(Pathname.new(path).cleanpath)
        true
      else
        _orig_exist?(path)
      end
    end

    alias :_orig_file? :file?
    def file?(path)
      path = path.to_s
      if #{self.class.name}::FILES.include?(Pathname.new(path).cleanpath)
        true
      else
        _orig_file?(path)
      end
    end

    alias :_orig_directory? :directory?
    def directory?(path)
      path = path.to_s
      if #{self.class.name}::DIRS.include?(Pathname.new(path).cleanpath)
        true
      else
        _orig_directory?(path)
      end
    end
  end
end

class ::IO
  class <<self
    alias :_orig_read :read
    def read(path, *args)
      path = path.to_s
      if content = #{self.class.name}::FILES[Pathname.new(path).cleanpath]
        content.to_default_external
      else
        _orig_read(path, *args)
      end
    end
  end
end

class ::Pathname
  alias :_orig_readable? :readable?
  def readable?
    if #{self.class.name}::FILES.include?(self.cleanpath)
      true
    else
      _orig_readable?
    end
  end
end

module ::FileUtils
  class <<self
    alias :_orig_mkdir_p :mkdir_p
    def mkdir_p(list, options = {})
      []
    end
  end
end
EOS

      yield

      eval <<EOS
class ::File
  class <<self
    alias :open :_orig_open
    alias :exist? :_orig_exist?
    alias :file? :_orig_file?
    alias :directory? :_orig_directory?
  end
end

class ::IO
  class <<self
    alias :read :_orig_read
  end
end

class ::Pathname
  alias :readable? :_orig_readable?
end

module ::FileUtils
  class <<self
    alias :mkdir_p :_orig_mkdir_p
  end
end
EOS
    end

    def self.name_to_rb_fname(name)
      "adlint/benchmark/target/#{normalize(name)}.rb"
    end
    private_class_method :name_to_rb_fname

    def self.name_to_rb_cname(name)
      "Target::T_#{normalize(name)}"
    end
    private_class_method :name_to_rb_cname

    def self.normalize(name)
      name.gsub("/", "-").gsub("-", "_").gsub(".", "_")
    end
    private_class_method :normalize
  end

end
end
